/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/



#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <string.h>
#include "ScatterWeb.Process.h"

uint8_t debug = 0;
/// Sequence number for send BANTs
uint16_t bant_sequence_num = 0;
/// Sequence number for send BANTs
uint16_t fant_sequence_num = 0;
/// Number of retransmissions of FANT
uint16_t resend_num = 0;
/// Size of incoming data packet
uint8_t size_of_data = 0;
/// Variables for loops etc
uint16_t i,j;
/// Variables for loops etc
uint8_t counter = 0;
uint8_t user_data = 0;
uint8_t cmdType = 0;
/// Variable for command BroadcastLED
uint8_t switchLED = 0;

/// String to store our ID
char idString[16];
/// Threshold for deleting an Routing Table entry
float pheromone_threshold = PHEROMONE_THRESHOLD;
/// Threshold for deleting an FANT/BANT Table entry
int age_threshold = AGE_THRESHOLD;
/// This value describes the decreasing of the pheromone value of a rotuing tabe entry
float pheromoneIncreasingValue = 0.8;	//
bool dump = true;
/// Variable to store the ID of the choosen entry in RT
uint8_t entry_id;  
/// Variables for testing loops
uint8_t loop_next_hop = 0; 
/// Variables for testing loops
uint8_t loop_destination = 0; 
/// Variables for testing (number of send packets)
uint16_t sending_counter = 0;
/// Variables for testing (number of received packets)
uint16_t receiving_counter = 0;

/// Variable to store the data of a packet
static char data_buffer [MAX_LENGTH_OF_DATA];

/// Function to cast int to ascii
char* my_itoa(uint8_t value, uint8_t length);

/// Function to send a FANT or BANT
uint8_t Send_FANT_BANT (struct data_of_fant_bant *p_fant_bant, uint8_t packet_type );
/// Function to add data of a FANT to the table
uint8_t Check_And_Add_FANT_table(struct data_of_fant_bant *p_fant);
/// Function to add data of a BANT to the table
uint8_t Check_And_Add_BANT_table(struct data_of_fant_bant *p_bant);
/// Function to create a Routing Table entry
uint8_t Add_Routing_Table_Entry(uint8_t destination_node, uint8_t next_hop, uint8_t flag, uint16_t hops);
/// Function for cleaning up the tables (delete entries)
void CleanUpTables(void* parameter);
/// Increase tables values (routing table, fant/bant table)
void IncreaseTableValues();

/// Function called, if a packet is received
bool Process_radioHandler(struct netpacket_handler_args* args)
{

	/*
		Notes: * The received packet data becomes invalid when this handler is completed.
	*/

	// Pass packet through command interface
	if ( Cmd_radioHandler( args ) ) {
		// packet handled by messaging module
		return true;
	}

	if ( args->cpaResult == ADRCOMP_OTHER ) {
		if (!dump){
			printf("From: %d\r\n",args->netheader->from);
			printf("To: %d\r\n",args->netheader->to);
			printf("ID: %u\r\n",Configuration.id);
		}
		// intercept packets for other nodes here
	} else if ( args->cpaResult == ADRCOMP_BROADCAST ) {
		if (!dump){
			printf("Broadcast From: %d\r\n",args->netheader->from);
			printf("Broadcast To: %d\r\n",args->netheader->to);
		}
		// process broadcast packets here
		struct data_of_fant_bant *p_fant;
		struct data_of_fant_bant *p_bant;
		
		
		switch ( args->netheader->type ) {
			
			///Process of FANT packet
			case USERDEFINED_PACKET+1:
			
			/*Structure of FANT packet
			
			|###########|-------------|-----------------|------------------|------|-----------|-------------|
			| Netheader | Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter | 
			|###########|-------------|-----------------|------------------|------|-----------|-------------|			
							uint8_t		    uint8_t		      uint8_t		uint8_t  uint16_t	 uint16_t
			*/
				
				p_fant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
		
				// Check if we got memory
				if (p_fant == NULL){
					printf("\r\n\r\nERROR: Could not allocate memory - p_fant");
					return false;
				}
				
				p_bant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
				// Check if we got memory
				if (p_bant == NULL){
					printf("\r\n\r\nERROR: Could not allocate memory - p_bant");
					return false;
				}
				
				
				printf("Received FANT from %d\r\n", args->netheader->from);
				//Get source node
				p_fant->source_node = *((uint8_t *) (args->payload));
				printf("source_node: %d ** ", p_fant->source_node);
				
				//Get pre_predecessor
				p_fant->pre_predecessor = *((uint8_t *) (args->payload) + 1);
				printf("pre_predecessor: %d ** ", p_fant->pre_predecessor);
				
				//Get Destination node
				p_fant->destination_node = *((uint8_t *) (args->payload) + 2);
				printf("dest_node: %d ** ", p_fant->destination_node);
				
				//Get Flag
				p_fant->flag = *((uint8_t *) (args->payload) + 3);
				printf("flag: %d ** ", p_fant->flag);
				
				//Get Sequence_Number
				p_fant->sequence_number = *((uint16_t *) (args->payload) + 2);
				printf("seq_number: %d ** ", p_fant->sequence_number);
				
				//Get Hop_Count
				p_fant->hop_counter = *((uint16_t *) (args->payload) + 3);
				printf("hop_counter: %d\r\n", p_fant->hop_counter);
				
				
				// For statistic -> count number of received FANTs
				receiving_counter++;
				printf("---NumOfFant %d\r\n", receiving_counter);
				
				if (p_fant->destination_node == Configuration.id){
					//We are the destination of the FANT packet
					struct data_of_fant_bant *p_bant;
					p_bant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
					// Check if we got memory
					if (p_bant == NULL){
						printf("\r\n\r\nERROR: Could not allocate memory - p_bant");
						return false;
					}
				
					// Create a BANT and send it the source of the FANT
					printf("I am the destination of this FANT, adding to FT&RT\r\n");
					Check_And_Add_FANT_table(p_fant);
					// Create a routing table entry
					Add_Routing_Table_Entry(p_fant->source_node, args->netheader->from, p_fant->flag, p_fant->hop_counter);
					p_bant->source_node = Configuration.id;
					p_bant->pre_predecessor = 0;
					p_bant->destination_node = p_fant->source_node;
					p_bant->flag = 0;
					p_bant->sequence_number = bant_sequence_num;
					bant_sequence_num++;					
					p_bant->hop_counter = 1;
					Check_And_Add_BANT_table(p_bant);
					Send_FANT_BANT(p_bant, PACKET_TYPE_BANT);
				}
				
				else if(p_fant->source_node == Configuration.id){
					//ignore
					printf("We have send this FANT\r\n");
				}
				//Make an entry in FANT table fant_table[SIZE_OF_FANT_TABLE]
				//check if we have already seen this packet
				else if (Check_And_Add_FANT_table(p_fant) == PACKET_IN_FANT_TABLE){
					//Do not forward			
					printf("FANT not added to FT\r\n");
					//Check if we have send this packet to our neighbor and it sends it back to us
					if (p_fant->pre_predecessor == Configuration.id){
						//ignore
						printf("We have send this FANT packet\r\n");
					}else{
						//Make entry in routing table
						//uint8_t destination_node, uint8_t next_hop, uint8_t flag
						printf("Make RT Entry\r\n");
						Add_Routing_Table_Entry(p_fant->source_node, args->netheader->from, p_fant->flag, p_fant->hop_counter);
					}
				}
				else {//we havent seen this packet
					//Make entry in routing table
					printf("Make RT Entry\r\n");
					Add_Routing_Table_Entry(p_fant->source_node, args->netheader->from, p_fant->flag, p_fant->hop_counter);
					
					//Change pre-predecessor of packet to actual predecessor
					p_fant->pre_predecessor = args->netheader->from;
					//increment hops
					p_fant->hop_counter++;
					//Forward FANT packet
					Send_FANT_BANT(p_fant, PACKET_TYPE_FANT);
				}
				
				// Free memory
				free(p_fant);
				free(p_bant);
				
				break;
			
			/// Handling of received BANT packet
			case USERDEFINED_PACKET+2:
			/*Structure of BANT packet
			
			|###########|-------------|-----------------|------------------|------|-----------|-------------|
			| Netheader | Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter | 
			|###########|-------------|-----------------|------------------|------|-----------|-------------|			
							uint8_t		    uint8_t		      uint8_t		uint8_t  uint16_t	 uint16_t
			*/
				
				p_fant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
		
				// Check if we got memory
				if (p_fant == NULL){
					printf("\r\n\r\nERROR: Could not allocate memory - p_fant");
					return false;
				}
				
				p_bant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
				// Check if we got memory
				if (p_bant == NULL){
					printf("\r\n\r\nERROR: Could not allocate memory - p_bant");
					return false;
				}

				
				printf("Received BANT from %d\r\n", args->netheader->from);
				//Get source node
				p_bant->source_node = *((uint8_t *) (args->payload));
				printf("source_node: %d ** ", p_bant->source_node);
				
				//Get pre_predecessor
				p_bant->pre_predecessor = *((uint8_t *) (args->payload) + 1);
				printf("pre_predecessor: %d ** ", p_bant->pre_predecessor);
				
				//Get Destination node
				p_bant->destination_node = *((uint8_t *) (args->payload) + 2);
				printf("dest_node: %d ** ", p_bant->destination_node);
				
				//Get Flag
				p_bant->flag = *((uint8_t *) (args->payload) + 3);
				printf("flag: %d ** ", p_bant->flag);
				
				//Get Sequence_Number
				p_bant->sequence_number = *((uint16_t *) (args->payload) + 2);
				printf("seq_number: %d ** ", p_bant->sequence_number);
				
				//Get Hop_Count
				p_bant->hop_counter = *((uint16_t *) (args->payload) + 3);
				printf("hop_counter: %d\r\n", p_bant->hop_counter);
				
				
				if (p_bant->destination_node == Configuration.id){
					//We are the destination of the BANT packet
					printf("I am the destination of this BANT, adding to BT&RT\r\n");
					Add_Routing_Table_Entry(p_bant->source_node, args->netheader->from, p_bant->flag, p_bant->hop_counter);
					Check_And_Add_BANT_table(p_bant);
					//Send data packet to receiver -> now we should have the next hop to the destination
					
				}
				else if(p_bant->source_node == Configuration.id){
					//ignore
					printf("We have send this BANT\r\n");
				}				
				//Make an entry in BANT table bant_table[SIZE_OF_BANT_TABLE]
				//check if we have already seen this packet
				else if (Check_And_Add_BANT_table(p_bant) == PACKET_IN_BANT_TABLE){
					//Do not forward			
					printf("BANT not added to BT\r\n");
					//Check if we have send this packet to our neighbor and it sends it back to us
					if (p_bant->pre_predecessor == Configuration.id){
						//ignore
					}else{
						//Make entry in routing table
						//uint8_t destination_node, uint8_t next_hop, uint8_t flag
						printf("Make RT Entry\r\n");
						Add_Routing_Table_Entry(p_bant->source_node, args->netheader->from, p_bant->flag, p_bant->hop_counter);
					}
				}
				else {//we havent seen this packet
					//Make entry in routing table
					printf("Make RT Entry\r\n");
					Add_Routing_Table_Entry(p_bant->source_node, args->netheader->from, p_bant->flag, p_bant->hop_counter);
					
					//Change pre-predecessor of packet to actual predecessor
					p_bant->pre_predecessor = args->netheader->from;
					//increment hops
					p_bant->hop_counter++;
					//Forward FANT packet
					Send_FANT_BANT(p_bant, PACKET_TYPE_BANT);
				}
				// Free memory
				free(p_fant);
				free(p_bant);
					
				break;
				
			case USERDEFINED_PACKET+9:
				/*Packet to toogle the LED*/
				cmdType = *((uint8_t *) (args->payload));
				printf("Received LED Broadcast\r\n");
				if (cmdType == 1) {
					do{ P5OUT ^= 0x80; }while(0);
				}
				break;
					
			
			default:
				break;
		}
		
	} else {
		// process data packets for this node
		struct data_packet * p_data;
		p_data = (struct data_packet*)malloc(sizeof(struct data_packet));
		
		// Check if we got memory
		if (p_data == NULL){
			printf("\r\n\r\nERROR: Could not allocate memory - p_data");
			return false;
		}
		
		uint8_t next_hop;

		switch ( args->netheader->type ) {
			case PING_REQUEST_PACKET:
				// send back to receiver's address
				Net_sendPong(args->netheader->from);
				break;

			case PING_REPLY_PACKET:
				LOG_LOW( "PONG from %u", args->netheader->from );
				break;

			/// Handling of data packets
			case USERDEFINED_PACKET+4:
			/*Structure of DATA packet
			
			|###########|-------------|------------------|-------------
			| Netheader | Source_Node | Destination_Node | Data ........
			|###########|-------------|------------------|-------------			
							uint8_t		    uint8_t		   uint8_t[uint8_t]
			*/
				
				
				// Get the data of the packet
				size_of_data = args->payload_size;
				// Source node
				p_data->source_node	  = *(uint8_t *)(args->payload);
				// Destination node
				p_data->destination_node = *((uint8_t *)(args->payload)+1);
				// Data
				p_data->data_point      = (uint8_t *)(args->payload)+2;
				// Add the id of this node to the data
				strcat(p_data->data_point, idString);
				
				
				printf("<<Data Packet Received from = %d, size = %d\r\n",args->netheader->from,strlen(p_data->data_point));
				printf("source = %d ",p_data->source_node);
				printf("destination = %d",p_data->destination_node);
				printf("Increasing RT\r\n");

				// Increase pheromone value
				Increase_Pheromone_Value (p_data->source_node, args->netheader->from, 1);
				
				// The data packet is this node
				if (p_data->destination_node == Configuration.id){
					receiving_counter++;
					printf("My data from %d is comming.\r\n",p_data->source_node);
					printf("data = %s --- Num %d\r\n",p_data->data_point, receiving_counter);
				}else{
					// Find the next hop
					next_hop = Find_Next_Hop (p_data->destination_node); 
					switch (next_hop){
						case 0:
							printf("no find data routing\r\n");
							break;
						default:
							Send_Data_Packet (p_data, next_hop);
							printf(">>send data packet to next hop %d\r\n",next_hop);
							break;
					}
				}
			break;
			
			default:
				return false;		// packet not handled (unknown)
		}
		return true;				// packet handled
	}
	return false;					// packet not handled (other destination)
}

void Process_sensorHandler(void* sdata)
{

}
/** 
* First called function -> initialise tables and data
*/
void Process_init()
{	

	uint8_t *p_clean_parameter;
	System_registerCallback( C_RADIO, ( fp_vp )Process_radioHandler );
	System_registerCallback( C_SENSOR, ( fp_vp )Process_sensorHandler );
	
	// For Testing -> create a String with our own ID
	memset (idString, '\0', sizeof(idString));
	idString[0] = '_';
	memcpy(idString + 1, my_itoa(Configuration.id, 2), 3);
	
	//Initialize routing table
	for (i = 0; i < SIZE_OF_ROUTING_TABLE; i++){
		routing_table[i].destination_node = 0;
		routing_table[i].next_hop = 0;
		routing_table[i].pheromone_value = 0;
		routing_table[i].flag = 0;
	}
	
	//Initialize fant table
	for(i=0; i < SIZE_OF_FANT_TABLE; i++){
		fant_table[i].source_node = 0;
		fant_table[i].sequence_number = 0;
		fant_table[i].aging_value = 0;
	}
	
	//Initialize bant table
	for(i=0; i < SIZE_OF_BANT_TABLE; i++){
		bant_table[i].source_node = 0;
		bant_table[i].sequence_number = 0;
		bant_table[i].aging_value = 0;
	}
	
	//Initialize statistic table for multipath counting
	for (i=0; i < STATISTIC_TABLE_NODES; i++){
		for (j=0; j < STATISTIC_TABLE_SIZE; j++){
			statistic_MultipathTable[i][j].destination = 0;
			statistic_MultipathTable[i][j].pathCounts = 0;
			//statistic_MultipathTable[i][j].p_data = NULL;
		}
	}

	//Start to clean the Tables (periodically)
	*p_clean_parameter = 1;		
	Timers_add(CLEAN_UP_TIMER*1024, CleanUpTables, (void *)&p_clean_parameter);
	Timers_add(INCREASING_TIMER*1024, IncreaseTableValues, (void *)0xFFFF);
}




/// Command to create & send a FANT packet with specific data
COMMAND(SendFANT, CMDFLAG_SERIAL, cmdargs) {
	struct data_of_fant_bant *p_fant = NULL;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	
	char* arg = (char*) cmdargs->args;
	if (count == 0){// Check if we have enough parameter
		printf("Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter \r\n");
		return;
	}
	
	// Parse the parameter and create FANT
	p_fant->source_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->pre_predecessor = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->destination_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->flag = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->sequence_number = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->hop_counter = atoi(arg);
	// Send the FANT
	Send_FANT_BANT(p_fant, PACKET_TYPE_FANT);
	
	return;
}
/// Command to create & send a BANT packet with specific data
COMMAND(SendBANT, CMDFLAG_SERIAL, cmdargs) {
	struct data_of_fant_bant *p_bant = NULL;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	
	char* arg = (char*) cmdargs->args;
	if (count == 0){
		printf("Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter \r\n");
		return;
	}
	
	// Parse the parameter and create FANT
	p_bant->source_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->pre_predecessor = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->destination_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->flag = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->sequence_number = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->hop_counter = atoi(arg);
	
	// Send BANT
	Check_And_Add_BANT_table(p_bant);
	Send_FANT_BANT(p_bant, PACKET_TYPE_BANT);
	
	return;
}

///Command to print the content of the Routing table
COMMAND(GetRT, CMDFLAG_SERIAL, cmdargs) {
	// Count args to remove compiler warning
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	if (count > 0 ){
		printf("No args needed");
	}
	
	printf("Routing Table of: %d\r\n ", Configuration.id);
	for (i=0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node != 0){			
			printf("Destination: %d ** ", routing_table[i].destination_node);
			printf("Next Hop: %d ** ", routing_table[i].next_hop);
			printf("Pheromone: %d ** ", (uint8_t)routing_table[i].pheromone_value);
			printf("Flag: %d\r\n", routing_table[i].flag);
		}
	}
	printf("\r\nStatistic\r\n");
	PrintStatisticMultipath();
}

///Command to print the FANT table
COMMAND(GetFT, CMDFLAG_SERIAL, cmdargs) {
	// Count args to remove compiler warning
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	if (count > 0 ){
		printf("No args needed");
	}
	
	for (i=0; i < SIZE_OF_FANT_TABLE; i++){
		if (fant_table[i].source_node != 0){			
			printf("Source of FANT: %d ** ", fant_table[i].source_node);
			printf("Sequence Num: %d \r\n", fant_table[i].sequence_number);
		}
	}
}

///Command to print the BANT table
COMMAND(GetBT, CMDFLAG_SERIAL, cmdargs) {
	// Count args to remove compiler warning
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	if (count > 0 ){
		printf("No args needed");
	}
	
	for (i=0; i < SIZE_OF_BANT_TABLE; i++){
		if (bant_table[i].source_node != 0){		
			printf("Source of BANT: %d ** ", bant_table[i].source_node);
			printf("Sequence Num: %d \r\n", bant_table[i].sequence_number);
		}
	}
}

///Command to create a routing table entry
COMMAND(CreateRTEntry, CMDFLAG_SERIAL, cmdargs) {
	struct routing_table_entry *p_rt = NULL;
	uint16_t hops;
	uint8_t counter, num,offset;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	
	char* arg = (char*) cmdargs->args;
	if (count == 0){
		printf("Destination | Next Hop | Flag | Hops\r\n");
		printf("2 parameter -> num of creating entries & starting value(offset nodeID)\r\n");
		return;
	}
	if (count == 2){
		num = atoi(arg);
		arg = ( char* )Cmd_GetNextArg( arg );
		offset = atoi(arg);
		for (counter = 0; counter < num; counter++){
			p_rt->destination_node = counter+offset;
			p_rt->next_hop = 2;
			p_rt->flag = 0;
			hops = 1;
			Add_Routing_Table_Entry(p_rt->destination_node, p_rt->next_hop, p_rt->flag, hops);
		}
		return;
	
	}
	p_rt->destination_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_rt->next_hop = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_rt->flag = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	hops = atoi(arg);
	
	Add_Routing_Table_Entry(p_rt->destination_node, p_rt->next_hop, p_rt->flag, hops);
}


//Command to broadcast the command to let the LEDs blink
COMMAND(BroadcastLED, CMDFLAG_SERIAL, cmdargs) {
	// Count args to remove compiler warning
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	if (count > 0 ){
		printf("No args needed");
	}
	if (switchLED == 0){
		switchLED = 1;
		printf("Start sending LED Broadcast");
		Timers_add(1024, SwitchLED, 0xFFFF);
	}
	else{
		switchLED = 0;
		printf("Stop sending LED Broadcast");
	}
	
}
/// Function to send periodically the LED on/off command
void SwitchLED(){

	char temp_data_buffer [MAX_LENGTH_OF_DATA+2];
	
	if (switchLED == 1){
		netpacket_send_args_t npsargs;
		Net_sendArgsInit(&npsargs);

		npsargs.netheader.to	  = 255;
		npsargs.netheader.type	  = USERDEFINED_PACKET+9;
		npsargs.netheader.flags   = 0;

		temp_data_buffer[0]			  = 1;
		temp_data_buffer[1]			  = '\0';
		
		npsargs.payload[0].buffer = temp_data_buffer;
		npsargs.payload[0].size   = strlen(temp_data_buffer)+1; // include terminating char
		
		Net_send(&npsargs);
		printf("Send LED Broadcast\r\n");

		Timers_add(1024, SwitchLED, 0xFFFF);
	}
}		



//----------------------------------------------------------------------------------------------------------------Send data packet for ara
/** Command to send a data packet
* The function will check if the routing table has a path to the destination node
* If yes -> Send packet to destination (next hop)
* If not -> Send a FANT and call a function which will try to resend the packet
*/
COMMAND (SendAraData, CMDFLAG_SERIAL, cmdargs) {

	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	if (count != 2){
		printf("Destination_Node | Data_String \r\n");
		return;
	}
	// Parse the parameter/arguments
	char* arg = (char*) cmdargs->args;
	uint8_t destination = (uint8_t) String_parseUint16(arg, NULL);
	uint8_t source = Configuration.id;
	struct data_of_fant_bant *p_fant = NULL;
	
	arg = ( char* )Cmd_GetNextArg( arg );

	// Set/Clean memory
	memset(data_buffer,'\0',MAX_LENGTH_OF_DATA);
	sprintf(data_buffer,"%s",arg);
	
	struct data_packet *p_data;
	uint8_t * p_data_point;
	
	// Allocate memory for data packet structure
	p_data = (struct data_packet*)malloc(sizeof(struct data_packet));
	
	// Allocate memory for data
	p_data_point = (uint8_t *)malloc(strlen(data_buffer));
	// Copy data
	memcpy(p_data_point, data_buffer, strlen(data_buffer)+1);
	
	// Set up data packet
	p_data->source_node	     = source;
	p_data->destination_node   = destination;
	p_data->data_point         = p_data_point;
	
	// Try to find the next hop in routing table
	uint8_t next_hop = Find_Next_Hop (p_data->destination_node); 
	switch (next_hop){
		case 0:
			// If we could not find a route
			printf("Could not find route\r\n");
			resend_num = 0;
			// Create FANT packet
			p_fant->source_node = source;
			p_fant->pre_predecessor = source;
			p_fant->destination_node = destination;
			p_fant->flag = 0;
			p_fant->sequence_number = fant_sequence_num;
			p_fant->hop_counter = 1;
			// Send FANT
			Check_And_Add_FANT_table(p_fant);
			Send_FANT_BANT(p_fant, PACKET_TYPE_FANT);
			// Increase sequence number of sent fants
			fant_sequence_num++;
			// Call method which will try to resend data packet
			Timers_add(1024*5,Resend_Data_Packet,(void *)p_data);
			break;
		default:
			// There is a route to the destination -> send packet
			Send_Data_Packet (p_data, next_hop);
			printf(">>Send data packet to next hop %d\r\n",next_hop);
			break;
	}
}

/// Command to sent many data packets (periodically)
COMMAND (SendManyAraData, CMDFLAG_SERIAL, cmdargs) {
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	if (count != 2){
		printf("Destination_Node | Data_String \r\n");
		return;
	}
	// Parse arguments
	char* arg = (char*) cmdargs->args;
	uint8_t destination = (uint8_t) String_parseUint16(arg, NULL);
	uint8_t source = Configuration.id;
	
	arg = ( char* )Cmd_GetNextArg( arg );

	// Set memory
	memset(data_buffer,'\0',MAX_LENGTH_OF_DATA);
	sprintf(data_buffer,"%s",arg);
	
	struct data_packet *p_data;
	uint8_t * p_data_point;
	
	// Allocate memory for data packet structure
	p_data = (struct data_packet*)malloc(sizeof(struct data_packet));
	
	// Allocate memory for data
	p_data_point = (uint8_t *)malloc(strlen(data_buffer));
	// Copy data
	memcpy(p_data_point, data_buffer, strlen(data_buffer)+1);
	
	// Create data packet
	p_data->source_node	     = source;
	p_data->destination_node   = destination;
	p_data->data_point         = p_data_point;
	
	// Find next hop to destination			
	loop_next_hop = Find_Next_Hop (p_data->destination_node); 
	loop_destination = p_data->destination_node;
	sending_counter = 0;
	switch (loop_next_hop){
		case 0:
			printf("Could not find route ABORT\r\n");
			break;
		default:
			Send_Data_Packet (p_data, loop_next_hop);
			// Call method which will send more packets
			Timers_add(256,Send_Many_Data_Packet,(void *)p_data);
			printf(">>Send data packet to next hop %d\r\n",loop_next_hop);
			break;
	}
}






//------------------------------------------------------------------------------------------------------------------------Method


/**
* Method to send a data packet
* \param data_packet	Pointer to the data packet which should be send
* \param next_hop		ID of the next node
* \return				0 if success
*/
uint8_t Send_Data_Packet (struct data_packet * data_packet, uint8_t next_hop) {
	
	// Store the values
	uint8_t   source		= data_packet->source_node;
	uint8_t	  destination   = data_packet->destination_node;
	uint8_t * data			= data_packet->data_point;
	char temp_data_buffer [MAX_LENGTH_OF_DATA+2];
	sprintf(temp_data_buffer+2,"%s",data);

	// Create a packet and send it
	netpacket_send_args_t npsargs;
	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to	  = next_hop;
	npsargs.netheader.type	  = USERDEFINED_PACKET+4;
	npsargs.netheader.flags   = 0;

	temp_data_buffer[0]			  = source;
	temp_data_buffer[1]			  = destination;

	npsargs.payload[0].buffer = temp_data_buffer;
	npsargs.payload[0].size   = strlen(temp_data_buffer)+1; // include terminating char
	
	Net_send(&npsargs);

	return 0;
}

/** Function to find the next hop in the routing table for a given destination
* \param	destination_node	ID of the destination of the sending packet
* \return	ID of the next hop, if no route exists -> 0
*/
uint8_t Find_Next_Hop (uint8_t destination_node){
	uint8_t i=0;
	///ID of the next hop for the destination
	uint8_t next_hop=0;
	/// Store how often the link was used
	uint8_t num_used = MAX_OF_UINT_8;
	/// Pherome value of the last entry
	float pheromone_value=0;
	printf("Searching in RT for route to %d\r\n", destination_node);

	//Decide which algorithm should be used
	if (ROUTE_CHOOSING_ALGORITHM == 0){
		for (;i<SIZE_OF_ROUTING_TABLE;i++){
			// Iterate through the routing table and find the entry whith highest pheromone value
			if (routing_table[i].destination_node == destination_node){
				if (routing_table[i].pheromone_value>pheromone_value){
					printf("Found Node: %d\r\n", routing_table[i].destination_node);
					next_hop		= routing_table[i].next_hop;
					pheromone_value = routing_table[i].pheromone_value;
					entry_id = i;
				}
			}
		}
	}
	
	//Decide which algorithm should be used
	if (ROUTE_CHOOSING_ALGORITHM == 1){
		// Round-Robin -> choose every route
		for (;i<SIZE_OF_ROUTING_TABLE;i++){
			if (routing_table[i].destination_node == destination_node){
				if (routing_table[i].flag < num_used){
					printf("Found Node: %d\r\n", routing_table[i].destination_node);
					next_hop		= routing_table[i].next_hop;
					pheromone_value = routing_table[i].pheromone_value;
					entry_id = i;
					num_used = routing_table[i].flag;
				}
			}
		}
		// Reset the value
		if (routing_table[entry_id].flag > 250)
			routing_table[entry_id].flag = 0;
		
	}
	// Return the next hop
	return next_hop;
}

/**
* Method to set the pheromone value of an entry to 0 (deactivate the link)
* \param destination	Destination node
* \param next hop		Next hop
* \return	0
*/
uint8_t Reset_Pheromone_Value (uint8_t destination, uint8_t next_hop){
	uint8_t i=0;
	for (;i<SIZE_OF_ROUTING_TABLE;i++){
		// Iterate through the table and find the entry
		if (routing_table[i].destination_node == destination){
			if (routing_table[i].next_hop==next_hop){
				// Set to 0 -> link will be deleted
				routing_table[i].pheromone_value = 0;
			}
		}
	}
	return 0;
}
/** Method to increase the pheromone values
* \param destination	Destination ID
* \param next_hop		ID of the next hop
* \param hops			Value for increasing the pheromone value (-> value += 1/hops)
* \return 0
*/
uint8_t Increase_Pheromone_Value (uint8_t destination, uint8_t next_hop, uint16_t hops){
	uint8_t i=0;
	for (;i<SIZE_OF_ROUTING_TABLE;i++){
		// Iterate through table
		if (routing_table[i].destination_node == destination){
			if (routing_table[i].next_hop==next_hop){
				// Increase value
				routing_table[i].pheromone_value += 1/hops;
			}
		}
	}
	return 0;
}


// resend the data packet (max 5 times) 
/** Function which will be called to try to resend a data packet. The methode will be called max 5 times -> max 5 retries
* \param data	Pointer to the packet which should be send
* \return 0
*/
uint8_t Resend_Data_Packet (void *data){

	struct data_packet * p_data = (struct data_packet *)data;
	
	printf("-------- resend data packet to %d ------------\r\n",p_data->destination_node);
	// Try to find a route
	uint8_t next_hop = Find_Next_Hop (p_data->destination_node);
		switch (next_hop){
		case 0:
			if (resend_num<5){
				// Could not find a route -> create & send FANT
				printf("Could not find route\r\n");
				struct data_of_fant_bant *p_fant = NULL;
				p_fant->source_node = p_data->source_node;
				p_fant->pre_predecessor = p_data->source_node;
				p_fant->destination_node = p_data->destination_node;
				p_fant->flag = 0;
				p_fant->sequence_number = fant_sequence_num;
				p_fant->hop_counter = 1;
				// Send FANT
				Check_And_Add_FANT_table(p_fant);
				Send_FANT_BANT(p_fant, PACKET_TYPE_FANT);
				// Increase FANT sequence number
				fant_sequence_num++;
				resend_num ++;
				// Call method to check, if can send the packet in some seconds
				Timers_add(1024*5,Resend_Data_Packet,(void *)data);
			}else {
				// reached max number of retries -> stop sending
				printf("Stop sending the packet again");
			}
			break;
		default:
			// Found a route -> Send data packet
			Send_Data_Packet (data, next_hop);
			printf(">>send data packet to next hop %d\r\n",next_hop);
			break;
	}
	return 0;
}

/** Method to send automatically many packets
* \param data	Pointer to the data packet
* \return 0
*/
uint8_t Send_Many_Data_Packet (void *data){
	loop_next_hop = Find_Next_Hop (loop_destination); 
	Send_Data_Packet (data, loop_next_hop);
	
	if (sending_counter < 1000){
		printf("Sending Num: %d\r\n", sending_counter);
		Timers_add(256,Send_Many_Data_Packet,(void *)data);
	}
	sending_counter ++;
	return 0;
}


/** Send (broadcast) a FANT/BANT packet to all nodes
* \param p_fant_bant	Pointer to data of fant or fant packet
* \param packet_type	Value to indicate whether we should send a FANT or BANT
* \return Success/Error-Code
*/
uint8_t Send_FANT_BANT (struct data_of_fant_bant *p_fant_bant, uint8_t packet_type ){

	/*
	|###########|-------------|-----------------|------------------|------|-----------|-------------|
	| Netheader | Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter | 
	|###########|-------------|-----------------|------------------|------|-----------|-------------|			
					uint8_t		    uint8_t		      uint8_t		uint8_t  uint16_t	 uint16_t
	*/
	
	uint16_t data_16[4]; 	//array for content of packet
	uint16_t byte_one = 0;	//bytes to concat two uint8_t values
	uint16_t byte_two = 0;
	
	byte_one = p_fant_bant->source_node;
	//shift to wright place for uint16_t
	byte_two = p_fant_bant->pre_predecessor << 8;	
	//concat source and pre-predecessor
	data_16[0] = byte_one | byte_two;	
	
	byte_one = p_fant_bant->destination_node;
	//shift to wright place for uint16_t
	byte_two = p_fant_bant->flag << 8;	
	//concat destination and flag
	data_16[1] = byte_one | byte_two;	
	//Sequence number
	data_16[2] = p_fant_bant->sequence_number;	
	//Hop Counter	
	data_16[3] = p_fant_bant->hop_counter;			
	
	netpacket_send_args_t npsargs;

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = 255; 		//Broadcast packet
	if (packet_type == PACKET_TYPE_FANT){//FANT
		npsargs.netheader.type = USERDEFINED_PACKET + PACKET_TYPE_FANT; // FANT
	}
	else if (packet_type == PACKET_TYPE_BANT){//BANT
		npsargs.netheader.type = USERDEFINED_PACKET + PACKET_TYPE_BANT; // BANT
	}
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = data_16;
	npsargs.payload[0].size = sizeof(data_16);
	
	Net_send(&npsargs);
	if (packet_type == PACKET_TYPE_FANT){//FANT
		printf("Send FANT\r\n");
	}
	else if (packet_type == PACKET_TYPE_BANT){//BANT
		printf("Send BANT\r\n");
	}

	return 0;
}

/** Function which checks if we have seen this FANT packet already, and add it to the fant table if not
* \param p_fant		Pointer to the FANT
* \return Returns PACKET_IN_FANT_TABLE if packet is in table; PACKET_NOT_IN_FANT_TABLE if not in table; FANT_TABLE_FULL if table is full
*/
uint8_t Check_And_Add_FANT_table(struct data_of_fant_bant *p_fant ){
	int first_free_pos = -1;
	
	// Iterate through the fant table
	for (i = 0; i < SIZE_OF_FANT_TABLE; i++){
		if (fant_table[i].source_node == p_fant->source_node
			&& fant_table[i].sequence_number == p_fant->sequence_number){
		
			// Found the sequence number and source id -> we have already seen this packet
			return PACKET_IN_FANT_TABLE;
		}
		// Store the first free position, if we have to add another
		if (fant_table[i].source_node == 0 && first_free_pos < 0){
			first_free_pos = i;
		}
	}
	// Check if the table is full
	if (first_free_pos > SIZE_OF_FANT_TABLE-1){
		return FANT_TABLE_FULL;
	}

	// Add the new FANT to the table
	fant_table[first_free_pos].source_node = p_fant->source_node;
	fant_table[first_free_pos].sequence_number = p_fant->sequence_number;
	return PACKET_NOT_IN_FANT_TABLE;
}

/** Function which checks if we have seen this BANT packet already, and add it to the bant table if not
* \param p_bant		Pointer to the BANT
* \return Returns PACKET_IN_BANT_TABLE if packet is in table; PACKET_NOT_IN_BANT_TABLE if not in table; FANT_TABLE_FULL if table is full
*/
uint8_t Check_And_Add_BANT_table(struct data_of_fant_bant *p_bant ){
	int first_free_pos = -1;
	
	// Iterate through the bant table
	for (i = 0; i < SIZE_OF_BANT_TABLE; i++){
		if (bant_table[i].source_node == p_bant->source_node
			&& bant_table[i].sequence_number == p_bant->sequence_number){
			// Found the sequence number and source id -> we have already seen this packet
			return PACKET_IN_BANT_TABLE;
		}
		if (bant_table[i].source_node == 0 && first_free_pos < 0){
			first_free_pos = i;
		}
	}
	// Check if the table is full
	if (first_free_pos > SIZE_OF_BANT_TABLE-1){
		return BANT_TABLE_FULL;
	}
	// Add the new BANT to the table
	bant_table[first_free_pos].source_node = p_bant->source_node;
	bant_table[first_free_pos].sequence_number = p_bant->sequence_number;
	return PACKET_NOT_IN_BANT_TABLE;
}



/** Adds a entry to the routing table, if an entry already exists the pheromone value will be increse
* \param destination_node 	Destination nodeID
* \param next_hop			Next hop to reach the destination node
* \param flag				flag for special use
* \param hops				Number of hops to reach the destination
* \return ROUTING_TABLE_ENTRY_EXISTS if entry already exists; ROUTING_TABLE_ENTRY_CREATED if a new entry is created; 
* ROUTING_TABLE_FULL if the oruting table is full
*/
uint8_t Add_Routing_Table_Entry(uint8_t destination_node, uint8_t next_hop, uint8_t flag, uint16_t hops){
	int first_free_pos = -1;
	float floatHops = (float)hops;
	printf("Adding to RT ...");
	for (i=0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node == destination_node
			&& routing_table[i].next_hop == next_hop){
			//Entry already in table
			// Update pheromone value
			routing_table[i].pheromone_value += (float)1/floatHops;
			routing_table[i].flag &= flag;
			printf("updated entry\r\n");
			return ROUTING_TABLE_ENTRY_EXISTS; 
		}
		// Store the first free position in table if we have to add an new entry
		if (routing_table[i].destination_node == 0 && first_free_pos < 0){
			first_free_pos = i;
		}
	}
	//Create a new table entry
	if (first_free_pos < SIZE_OF_ROUTING_TABLE-1){
		printf("created new entry\r\n");
		routing_table[first_free_pos].destination_node = destination_node;
		routing_table[first_free_pos].next_hop = next_hop;
		// Calculate the pheromone value
		routing_table[first_free_pos].pheromone_value = (float)1/floatHops;
		routing_table[first_free_pos].flag = flag;
		return ROUTING_TABLE_ENTRY_CREATED;
	}
	
	//The table is full
	if (first_free_pos == SIZE_OF_ROUTING_TABLE-1)
		printf("RT full\r\n");
		return ROUTING_TABLE_FULL;
		
	return 0;
}

/** Method which iterates through all tables (fant/bant, routing table) and deletes all unused entries (Threshold)
*/
void CleanUpTables(void* parameter){

	uint8_t *p_recall = (uint8_t*)parameter;
	uint16_t num_used_RT = 0;
	uint16_t num_used_FT = 0;
	uint16_t num_used_BT = 0;
	
	if (debug == 1)
		printf("Cleaning up Routing table ... ");
	// Iterate througt routing table and delete all entries where pheromone <= pheromone_threshold
	for (i=0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node != 0){
			num_used_RT++;
			if (routing_table[i].pheromone_value <= pheromone_threshold){
				// Delete entry
				routing_table[i].destination_node = 0;
				routing_table[i].next_hop = 0;
				routing_table[i].pheromone_value = 0;
				routing_table[i].flag = 0;
			}
		}
	}
	
	if (debug == 1)
		printf("Cleaning up FANT/BANT table\r\n");
	// Iterarte through FANT table and delete entries where aging_value >= age_threshold
	for(i=0; i < SIZE_OF_FANT_TABLE; i++){
		if (fant_table[i].source_node != 0){
			num_used_FT++;
			if (fant_table[i].aging_value >= age_threshold){
				fant_table[i].source_node = 0;
				fant_table[i].sequence_number = 0;
				fant_table[i].aging_value = 0;
			}
		}
	}
	// Iterarte through BANT table and delete entries where aging_value >= age_threshold
	for(i=0; i < SIZE_OF_BANT_TABLE; i++){
		if (bant_table[i].source_node != 0){
			num_used_BT++;
			if (bant_table[i].aging_value >= age_threshold){
				bant_table[i].source_node = 0;
				bant_table[i].sequence_number = 0;
				bant_table[i].aging_value = 0;
			}
		}
	}
	//Check num of routing table entries and adjust threshold
	if (num_used_RT >= SIZE_OF_ROUTING_TABLE-10){
		// We have just some free entries -> set down threshold
		pheromone_threshold++;
	}
	else{
		pheromone_threshold = 0;
	}
	
	//Check num of fant/bant table entries and adjust threshold
	if (num_used_FT >= SIZE_OF_FANT_TABLE-10 || num_used_BT >= SIZE_OF_BANT_TABLE-10){
		if (age_threshold > 0)
			age_threshold--;
	}
	else{
		age_threshold = AGE_THRESHOLD;
	}
	
	// Reset values for sequence number etc
	if (fant_sequence_num > 4000)
		fant_sequence_num = 0;
	if (bant_sequence_num > 4000)
		bant_sequence_num = 0;
		

	//Check, if table is full - recall clean up immediately again
	if (num_used_RT >= SIZE_OF_ROUTING_TABLE-5 || num_used_FT >= SIZE_OF_FANT_TABLE-5 || num_used_BT >= SIZE_OF_BANT_TABLE-5){
		Timers_add(512, CleanUpTables, parameter);
	}
	else if (*p_recall != 0){// Recall the cleaning method
		Timers_add(CLEAN_UP_TIMER*1024, CleanUpTables, parameter);
	}
	
}


/**Decreases periodically the pheromone value of the routing table and increases the values in fant and bant table
*/
void IncreaseTableValues(){
	
	if (debug == 1)
		printf("Increasing Routing table (pheromone) ... ");
	
	// Iterate through Routing Table
	for (i=0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node != 0){
			//routing_table[i].pheromone_value *= pheromoneIncreasingValue;
		}
	}
	
	// Iterate through FANT and BANT-Table
	if (debug == 1)
		printf("decreasing fant/bant table\r\n");
	for(i=0; i < SIZE_OF_FANT_TABLE; i++){
		if (fant_table[i].source_node != 0){
			fant_table[i].aging_value++;
		}
	}
	for(i=0; i < SIZE_OF_BANT_TABLE; i++){
		if (bant_table[i].source_node != 0){
			bant_table[i].aging_value++;
		}
	}
	// Recall this method
	Timers_add(INCREASING_TIMER*1024, IncreaseTableValues, (void *)0xFFFF);

}

/** For statistic: Store all received data packets for statistic
* \param data	Pointer data packet
*/
void UpdateMultipathTable(char *data){
	uint8_t statisticEntryExists = 0;
	uint8_t firstFreePos = STATISTIC_TABLE_SIZE;
	
	
	if (statisticMultipathTableColumn == STATISTIC_TABLE_SIZE){
		printf ("STATISTIC TABLE IS FULL\r\n\r\n");
		return;
	}
	
	// We have a 2D array, store number of paths to destination and the received packet in a column
	for (i = 0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node != 0){
			statisticEntryExists = 0;
			for (j=0; j < STATISTIC_TABLE_NODES; j++){
				if (statistic_MultipathTable[j][statisticMultipathTableColumn].destination == routing_table[i].destination_node){
					statistic_MultipathTable[j][statisticMultipathTableColumn].pathCounts++;
					//memcpy(statistic_MultipathTable[statisticMultipathTableColumn][j].data, data, strlen(data) + 1);
					statisticEntryExists = 1;
					break;
				}
				if (statistic_MultipathTable[j][statisticMultipathTableColumn].destination == 0){
					firstFreePos = j;
					break;
				}
				
				
			}
			if (statisticEntryExists == 0){
				statistic_MultipathTable[firstFreePos][statisticMultipathTableColumn].destination = routing_table[i].destination_node;
				statistic_MultipathTable[firstFreePos][statisticMultipathTableColumn].pathCounts = 1;
				printf("---Data for StatisticTable--->%s\r\n",data);
				if (strlen(*data) + 1 < STATISTIC_TABLE_STR_SIZE)
					memcpy(statistic_MultipathTable[firstFreePos][statisticMultipathTableColumn].p_data, data, strlen(data) + 1);
			}
		}
	}
	statisticMultipathTableColumn++;
}
	
/** Print out the statistic table
*/
void PrintStatisticMultipath(){
	
	for (i = 0; i < STATISTIC_TABLE_NODES; i++){
		for (j = 0; j < STATISTIC_TABLE_SIZE; j++){
			printf("|%d-%d-%s|", statistic_MultipathTable[i][j].destination, statistic_MultipathTable[i][j].pathCounts, statistic_MultipathTable[i][j].p_data);
		}
		printf("\r\n");
	}

}
/** Convert int to char
* \param value		Int value to convert
* \param length 	Number of digits
* \return char
*/
char *my_itoa(uint8_t value, uint8_t length) {
   char *returnString =(char *) malloc(length+1 * sizeof(char));
   uint8_t i;
   for(i  =0; i < length; i++) {
      returnString[length-i-1] = (value % 10) + 48;
      value = value / 10;
   }
   returnString[length]='\0';
   return returnString;
}


// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);

