#define MAX_OF_UINT_8				255


// Define return values for fant/bant table operations
#define PACKET_IN_FANT_TABLE		0
#define PACKET_NOT_IN_FANT_TABLE	10
#define FANT_TABLE_FULL				11
#define PACKET_IN_BANT_TABLE		0
#define PACKET_NOT_IN_BANT_TABLE	10
#define BANT_TABLE_FULL				11

// Define return values for routing table operations
#define ROUTING_TABLE_ENTRY_CREATED	0
#define ROUTING_TABLE_ENTRY_EXISTS	0
#define ROUTING_TABLE_FULL			20

// Define the algorithm which finds the next hop in the routing table
// 0 -> Choose route with highest pheromone value
// 1 -> Round Robin
#define ROUTE_CHOOSING_ALGORITHM	0

// Define size of tables
#define SIZE_OF_ROUTING_TABLE	100
#define SIZE_OF_FANT_TABLE		100
#define SIZE_OF_BANT_TABLE		100
#define STATISTIC_TABLE_SIZE	3
#define STATISTIC_TABLE_NODES	3
#define STATISTIC_TABLE_STR_SIZE	25

// Define packet types
#define PACKET_TYPE_FANT		1
#define PACKET_TYPE_BANT		2

// Define timer and thresholds for increasing/decreasing and deleting entries
#define CLEAN_UP_TIMER			30
#define INCREASING_TIMER		20
#define PHEROMONE_THRESHOLD		0.2
#define AGE_THRESHOLD			5

// The max length of the data packet
#define MAX_LENGTH_OF_DATA      100 
#define WAITING_ACK_1           1
#define WAITING_ACK_2			2
#define WAITING_ACK_3			3

/// Routing table entry
struct routing_table_entry{
	uint8_t destination_node;		// ID of destination node
	uint8_t next_hop;				// NodeID of the next node
	float pheromone_value;			// Value of the pheromone for the route
	uint8_t flag;					// Additional flag to store some data
};


/// Data of a BANT or FANT packet
struct data_of_fant_bant{
	uint8_t source_node;			// ID of the source node (sending the FANT)
	uint8_t destination_node;		// ID of the destination node
	uint16_t sequence_number;		// Sequence number: Number of FANTs or BANTs of the source node (incremental)
	uint16_t hop_counter;			// Number of hops of the packet
	uint8_t pre_predecessor;		// Node ID of the pre-predecessor
	uint8_t flag;					// Additional flag to store data (e.g. if BANT or FANT)
};


/// Structure of FANT table to identify duplicated FANT packets
struct fant_bant_table_entry{
	uint8_t source_node;			// ID of the source node (sending the FANT)
	int aging_value;				// Increment every CLEAN_UP_TIMER seconds this value, if it is to big - delete entry
	uint16_t sequence_number;		// Sequence number: Number of FANTs or BANTs of the source node (incremental)
};



/// Routing Table
static struct routing_table_entry routing_table[SIZE_OF_ROUTING_TABLE];
/// FANT Table
static struct fant_bant_table_entry fant_table[SIZE_OF_FANT_TABLE];
/// BANT Table
static struct fant_bant_table_entry bant_table[SIZE_OF_BANT_TABLE];



/// Statistic: Struct to store received pakcets and the number of multipaths
struct statistic_MultipathTable{
	uint8_t destination;
	uint8_t pathCounts;
	uint8_t p_data[STATISTIC_TABLE_STR_SIZE];
};
/// Atatistic table
static struct statistic_MultipathTable statistic_MultipathTable[STATISTIC_TABLE_NODES][STATISTIC_TABLE_SIZE];
static uint8_t statisticMultipathTableColumn = 0;
//-----------------------------------------------------------------------------------------------data packet


/// Structure of a Data packet
struct data_packet{
	uint8_t source_node;			// ID of the source node 
	uint8_t destination_node;		// ID of the destination node
	uint8_t * data_point;			// point to store data (String)
};
/// send the data packet to the next hop.
uint8_t Send_Data_Packet (struct data_packet * data_packet, uint8_t next_hop) ;
/// find the next hop in routing table
uint8_t Find_Next_Hop (uint8_t destination_node);
/// reset the pheromone value of this entry in RT to 0. Entry of the RT is identified by destination and next_hop
uint8_t Reset_Pheromone_Value (uint8_t destination, uint8_t next_hop);
/// increase the pheromone value of this entry in RT.  Entry of the RT is identified by destination and next_hop
uint8_t Increase_Pheromone_Value (uint8_t destination, uint8_t next_hop, uint16_t hops);
/// resend the data packet after 5 second
uint8_t Resend_Data_Packet ();
/// Send a broadcast to switch on/off the LED
void SwitchLED();
/// Save the multipath information in the Statistic Table
void UpdateMultipathTable(char *data);
/// Print out the statistic table
void PrintStatisticMultipath();

uint8_t Send_Many_Data_Packet (void *data);