/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <string.h>

struct TrafficGeneratorSettings{
	float interval;
	uint8_t limit;
	uint8_t status;
	uint8_t frame_size;
}trafficSettings;

uint8_t counter = 0;
uint8_t user_data = 0;
bool dump = false;
uint8_t destination = 5;

//Method
void callMe();
void blinkBlank(void* loops);
void trafficGenerator();

bool Process_radioHandler(struct netpacket_handler_args* args)
{

	
	/*
		Notes: * The received packet data becomes invalid when this handler is completed.
	*/

	// Pass packet through command interface
	if ( Cmd_radioHandler( args ) ) {
		// packet handled by messaging module
		return true;
	}

	if ( args->cpaResult == ADRCOMP_OTHER ) {
		if (!dump){
			printf("From: %d\r\n",args->netheader->from);
			printf("To: %d\r\n",args->netheader->to);
			printf("ID: %u\r\n",Configuration.id);
		}
		// intercept packets for other nodes here
	} else if ( args->cpaResult == ADRCOMP_BROADCAST ) {
		if (!dump){
			printf("Broadcast From: %d\r\n",args->netheader->from);
			printf("Broadcast To: %d\r\n",args->netheader->to);
		}
		// process broadcast packets here
	} else {
		// process packets for this node here
		switch ( args->netheader->type ) {
			case PING_REQUEST_PACKET:
				// send back to receiver's address
				Net_sendPong(args->netheader->from);
				break;

			case PING_REPLY_PACKET:
				LOG_LOW( "PONG from %u", args->netheader->from );
				break;

			case USERDEFINED_PACKET+4:
				if (!dump){
					user_data = *((uint8_t *) (args->payload));
					user_data = user_data * 2;
					printf("UserData: %d\r\n", user_data);
					printf("From: %d\r\n",args->netheader->from);
					//blinkBlank ((void *)&user_data);
				}
				
			break;

			default:
				return false;		// packet not handled (unknown)
		}
		return true;				// packet handled
	}
	return false;					// packet not handled (other destination)
}

void Process_sensorHandler(void* sdata)
{

}

void Process_init()
{
	System_registerCallback( C_RADIO, ( fp_vp )Process_radioHandler );
	System_registerCallback( C_SENSOR, ( fp_vp )Process_sensorHandler );

	//Call the callMe function 5s after system start
	//Timers_add(5*1024, callMe, 0xFFFF);
}

/**
 * @brief	Ping request
 *
 * Send a ping request to another node
 * Example: ping 24
 */
COMMAND( ping, 0, cmdargs )
{
	uint16_t node;

	node = String_parseUint16( cmdargs->args, NULL );
	if ( node > 0 ) {
		Net_sendPing((netaddr_t)node);
	}
}

/**
 * @brief Echo Arguments
 *
 * Testcase 1: long command name
 * Testcase 2: argument splitting
 */
COMMAND( echoargs, CMDFLAG_SERIAL, cmdargs )
{
	uint16_t i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;

	for ( i = 1; i <= count; i++ ) {
		printf("%u: %*s\n", i, strlen(arg), arg);
		arg = ( char* )Cmd_GetNextArg( arg );
	}
	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "%u args found", count );
}

// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);




// Eigene Commands

COMMAND(HelloMSB, CMDFLAG_SERIAL, cmdargs) {
	
	uint16_t n = String_parseUint16(cmdargs->args, NULL);

	while(n--) {
		printf("Hello User called %u times\r\n", counter++);
	}
}

COMMAND(SMyP, CMDFLAG_SERIAL, cmdargs) {
	
	netpacket_send_args_t npsargs;
	uint16_t data = (uint8_t) String_parseUint16(cmdargs->args, NULL);

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = 4; // choose a destination id
	npsargs.netheader.type = USERDEFINED_PACKET+4; // choose a number
	npsargs.netheader.flags = 0;


	npsargs.payload[0].buffer = &data;
	npsargs.payload[0].size = sizeof(data); // include terminating char

	Net_send(&npsargs);

}


COMMAND(genPackets, CMDFLAG_SERIAL, cmdargs) {
	uint16_t i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;
	float interval_1 = 0;
	float interval_2 = 0;
	float interval = 0;
	float tmp_float = 1;
	char * p_point = NULL;
	uint16_t limit = 0;
	uint8_t command = 0; //0 - Stop; 1 - Start
	
	/*First argument - Start/Stop*/
	if (strstr(arg, "start")){
		command = 1;
		trafficSettings.status = 1;
	}
	else if (strstr(arg, "stop")){
		command = 0;
		trafficSettings.status = 0;
		printf("Sending stopped.\r\n");
		return;
	}
	else {
		printf("Please enter correct parameters [1].\r\n");
		return;
	}

	/*Second argument - Interval*/
	arg = ( char* )Cmd_GetNextArg( arg );
	printf("second arg %s\r\n", arg);
	p_point = strtok(arg, ".");
	if (p_point == NULL)
		printf("Please enter correct parameters [2].\r\n");
	interval_1 = atoi(p_point);
	
	p_point = strtok( NULL, ".");
	if (p_point == NULL)
		interval_2 = 0;
	else{
		arg = ( char* )Cmd_GetNextArg( arg ); //We changed the argument string - jump to next arg
		for (i=0; i<=strlen(p_point);i++){
			tmp_float = 10.0 * tmp_float;
		}
		interval_2 = atoi(p_point);
		//printf("atoi(p_point) %d\r\n", atoi(p_point));
		interval_2 = interval_2/tmp_float;
		
	}
	interval = interval_1+interval_2;
	
	/*Third argument - Limit*/
	arg = ( char* )Cmd_GetNextArg( arg );
	//printf("third arg %s\r\n", arg);
	limit = atoi(arg);
	//printf("limit %d\r\n", limit);

	/*Fourth argument - Destination*/
	if (count>=4) {
		arg = ( char* )Cmd_GetNextArg( arg );
		destination = atoi (arg);
	}
	/*Fifth argument - Destination*/
	if (count >= 5){
		arg = ( char* )Cmd_GetNextArg( arg );
		trafficSettings.frame_size = atoi (arg);
	}
	

	/*Send the packets*/
	trafficSettings.limit = limit;
	trafficSettings.interval = interval;
	//p_trafficSettings = &trafficSettings;
	//printf("trafficSettings.limit %d\r\n", trafficSettings.limit);
	//printf("p_trafficSettings.limit %d\r\n", p_trafficSettings->limit);
	
	Timers_add(256, trafficGenerator, 0xFFFF);
}

COMMAND(Dump, CMDFLAG_SERIAL, cmdargs) {
	
	if (strstr ((char*) cmdargs->args,"start")){
		dump = true;
		printf("Dumping: On");
	}
	if (strstr ((char*) cmdargs->args,"stop")){
		dump = false;
		printf("Dumping: Off");
	}
}

//Method
void callMe(){

	printf("Blink\r\n");

	do{ P5OUT ^= 0x80; }while(0);
	//Timers_add(2*1024, callMe, 0xFFFF);
}

void blinkBlank(void* loops){

	uint8_t *p_loops = (uint8_t*)loops;
	do{ P5OUT ^= 0x80; }while(0);
	printf("Loop: %d\r\n", *p_loops);
	
	if (--(*p_loops)){
		
		Timers_add(256, blinkBlank, loops);
	}
}


void trafficGenerator(){
	
	uint8_t data [trafficSettings.frame_size];

	netpacket_send_args_t npsargs;

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = destination; // choose a destination id
	npsargs.netheader.type = USERDEFINED_PACKET+4; // choose a number
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = data;
	npsargs.payload[0].size = sizeof(data); // include terminating char
	printf("Packet_Size %d - trafficsetting %d\r\n",sizeof(data), trafficSettings.frame_size);
	
	do{ P5OUT ^= 0x80; }while(0);
	Net_send(&npsargs);

	if ((--trafficSettings.limit) && (trafficSettings.status == 1)){
				
		printf("genPackets limit %d\r\n",trafficSettings.limit);
		//Timers_add(p_trafficSettings->interval*1024, trafficGenerator, (void *) p_trafficSettings);
		Timers_add(1024.0*trafficSettings.interval, trafficGenerator, 0xFFFF);
	}
	
}