/*
Copyright 2008, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2008
*/

/**
 * @file			ScatterWeb.SensorBoard.c
 * @ingroup	libSensorBoard
 * @brief			SensorBoard library
 * 
 * @author		Marco Hutta
 * @date			Jan 2008
 * @since		1.2
 * @version	$Revision$
 * 
 */

#include "ScatterWeb.SensorBoard.internals.h"
#include "ScatterWeb.SensorBoard.h"

interrupt(NOVECTOR) _SensorBoard_portOne() {
    volatile int n = P1IFG & P1IE;
    if (n & (1 << 0)) {
			P1IFG &= ~(1 << 0);
      Data_sensorEventValue.board = Data_boardID;
			Data_sensorEventValue.channel = 10;
			Data_sensorEventValue.timeStamp = CURRENT_TIME;
			Data_sensorEventValue.value_length = 1;
			Data_sensorEventValue.value = Data_sensorEventStream;
			Data_sensorEventStream[0] = 1;
			WAKEUPLPM1(MF_SENSOR);
      return;
    }
    if (n & (1 << 1)) {
			P1IFG &= ~(1 << 1);
      Data_sensorEventValue.board = Data_boardID;
			Data_sensorEventValue.channel = 11;
			Data_sensorEventValue.timeStamp = CURRENT_TIME;
			Data_sensorEventValue.value_length = 1;
			Data_sensorEventValue.value = Data_sensorEventStream;
			Data_sensorEventStream[0] = 1;
			WAKEUPLPM1(MF_SENSOR);
      return;
    }
		if (n & (1 << 2)) {
		  P1IFG &= ~(1 << 2);
      Data_sensorEventValue.board = Data_boardID;
			Data_sensorEventValue.channel = 12;
			Data_sensorEventValue.timeStamp = CURRENT_TIME;
			Data_sensorEventValue.value_length = 1;
			Data_sensorEventValue.value = Data_sensorEventStream;
			Data_sensorEventStream[0] = 1;
			WAKEUPLPM1(MF_SENSOR);
      return;
    }
}

void _SensorBoard_redOn() {
  P4OUT &= 0xFE;
}

void _SensorBoard_yellowOn() {
  P4OUT &= 0xFD;
}

void _SensorBoard_greenOn() {
  P4OUT &= 0xFB;
}

void _SensorBoard_beeperOn() {
  P4OUT &= 0xF7;
}

void _SensorBoard_firstCollectorOn() {
}

void _SensorBoard_secondCollectorOn() {
}

void _SensorBoard_thirdCollectorOn() {
}

void _SensorBoard_fourthCollectorOn() {
}

void _SensorBoard_firstButtonOn() {
  P1IES |= (1 << 0);
  P1IFG &= ~(1 << 0);
  P1IE |= (1 << 0);
}

void _SensorBoard_secondButtonOn() {
  P1IES |= (1 << 1);
  P1IFG &= ~(1 << 1);
  P1IE |= (1 << 1);
}

void _SensorBoard_movementOn() {
  P1IES |= (1 << 2);
  P1IFG &= ~(1 << 2);
  P1IE |= (1 << 2);
}

void _SensorBoard_redOff() {
  P4OUT |= ~0xFE;
}

void _SensorBoard_yellowOff() {
  P4OUT |= ~0xFD;
}

void _SensorBoard_greenOff() {
  P4OUT |= ~0xFB;
}

void _SensorBoard_beeperOff() {
  P4OUT |= ~0xF7;
}

void _SensorBoard_firstCollectorOff() {
}

void _SensorBoard_secondCollectorOff() {
}

void _SensorBoard_thirdCollectorOff() {
}

void _SensorBoard_fourthCollectorOff() {
}

void _SensorBoard_firstButtonOff() {
  P1IES &= ~(1 << 0);
  P1IFG &= ~(1 << 0);
  P1IE &= ~(1 << 0);
}

void _SensorBoard_secondButtonOff() {
  P1IES &= ~(1 << 1);
  P1IFG &= ~(1 << 1);
  P1IE &= ~(1 << 1);
}

void _SensorBoard_movementOff() {
  P1IES &= ~(1 << 2);
  P1IFG &= ~(1 << 2);
  P1IE &= ~(1 << 2);
}

bool _SensorBoard_redLed() {
  return !(P4OUT & 0x01);
}

bool _SensorBoard_yellowLed() {
  return !(P4OUT & 0x02);
}

bool _SensorBoard_greenLed() {
  return !(P4OUT & 0x04);
}

bool _SensorBoard_beeper() {
  return !(P4OUT & 0x08);
}

bool _SensorBoard_firstCollector() {
 return !False;
}

bool _SensorBoard_secondCollector() {
 return !False;
}

bool _SensorBoard_thirdCollector() {
 return !False;
}

bool _SensorBoard_fourthCollector() {
 return !False;
}

void _SensorBoard_enable(void* line) {
  uint8_t channel = *(uint8_t*) line;
  if(channel == 2) _SensorBoard_redOn();
  if(channel == 3) _SensorBoard_yellowOn();
  if(channel == 4) _SensorBoard_greenOn();
  if(channel == 5) _SensorBoard_beeperOn();
  if(channel == 6) _SensorBoard_firstCollectorOn();
  if(channel == 7) _SensorBoard_secondCollectorOn();
  if(channel == 8) _SensorBoard_thirdCollectorOff();
  if(channel == 9) _SensorBoard_fourthCollectorOn();
  if(channel == 10) _SensorBoard_firstButtonOn();
  if(channel == 11) _SensorBoard_secondButtonOn();
  if(channel == 12) _SensorBoard_movementOn();
}

void _SensorBoard_diable(void* line) {
  uint8_t channel = *(uint8_t*) line;
  if(channel == 2) _SensorBoard_redOff();
  if(channel == 3) _SensorBoard_yellowOff();
  if(channel == 4) _SensorBoard_greenOff();
  if(channel == 5) _SensorBoard_beeperOff();
  if(channel == 6) _SensorBoard_firstCollectorOff();
  if(channel == 7) _SensorBoard_secondCollectorOff();
  if(channel == 8) _SensorBoard_thirdCollectorOff();
  if(channel == 9) _SensorBoard_fourthCollectorOff();
  if(channel == 10) _SensorBoard_firstButtonOff();
  if(channel == 11) _SensorBoard_secondButtonOff();
  if(channel == 12) _SensorBoard_movementOff();
}

void _SensorBoard_read(void* line) {
  uint8_t channel = *(uint8_t*) line;
  if(channel == 2) {
    Data_sensorValue.channel = 2;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_redLed() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 3) {
    Data_sensorValue.channel = 3;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_yellowLed() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 4) {
    Data_sensorValue.channel = 4;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_greenLed() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 5) {
    Data_sensorValue.channel = 5;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_beeper() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 6) {
    Data_sensorValue.channel = 6;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_firstCollector() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 7) {
    Data_sensorValue.channel = 7;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_secondCollector() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 8) {
    Data_sensorValue.channel = 8;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_thirdCollector() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
  if(channel == 9) {
    Data_sensorValue.channel = 9;
	  Data_sensorValue.value_length = 1;
	  uint8_t temp = _SensorBoard_fourthCollector() ? 1 : 0;
	  Data_sensorValue.timeStamp = CURRENT_TIME;
	  Data_sensorStream[0] = temp;
	  Data_sensorValue.value = Data_sensorStream;
  }
}

/******************************************************************************
 * @name	Initialization and configuration
 * @{
 */

void libSensorBoard_Init() {
}

void SensorBoard_init() {
	SET_ISR(PORT1_VECTOR, _SensorBoard_portOne);
	Data_registerDriver(_SensorBoard_enable, _SensorBoard_diable, _SensorBoard_read);
	P1SEL = 0x00;
	P1OUT = 0x00;
	P1DIR = 0x00;
	P4SEL = 0x00;
	P4OUT = 0x0F;
	P4DIR = 0x0F;
	Data_boardID = 2;
	Data_channels = 13;
}

/** @} */
